Imports System.Net
Imports System.Net.Sockets
Imports System.Windows.Forms

Public Class clsTCPSocket

    Private Const VERSION_NO As String = "1.01.0001"

    Public Enum eState
        Closed = 1
        Connecting
        Connected
        Listening
    End Enum

#Region "Variables"
    'events 
    Public Event IncomingConnection()
    Public Event Connected()
    Public Event Connecting()
    Public Event Closed()
    Public Event DataArrival(ByVal pBytes() As Byte, ByVal pLength As Long)
    Public Event Disconnected()

    'objects
    Private mSocket As Socket
    Private mListener As TcpListener
    Private WithEvents mListenerTimer As Timer
    Private WithEvents mDataTimer As Timer

    'variables
    Private mHostName As String
    Private mHostIP As String    
    Private mPort As Integer
    Private mState As eState
#End Region

    Public Sub New(Optional ByVal pPort As Integer = 0)
        mDataTimer = New Timer
        mDataTimer.Interval = 1000
        mDataTimer.Stop()

        mListenerTimer = New Timer
        mListenerTimer.Interval = 1000
        mListenerTimer.Stop()

        mState = eState.Closed
        mPort = pPort
    End Sub

    Protected Overrides Sub Finalize()
        Try
            mDataTimer.Dispose()
            mListenerTimer.Dispose()

            If Not (mSocket Is Nothing) Then mSocket = Nothing
            If Not (mListener Is Nothing) Then mListener = Nothing

            MyBase.Finalize()
        Catch ex As Exception
        End Try
    End Sub

#Region "Properties"

    Public ReadOnly Property Version() As String
        Get
            Return VERSION_NO
        End Get
    End Property
    Public ReadOnly Property LocalName() As String
        Get
            Return Dns.GetHostName
        End Get
    End Property

    Public ReadOnly Property LocalIP() As String
        Get
            Return Dns.Resolve(Dns.GetHostName).AddressList(0).ToString
        End Get
    End Property

    Public Property Port() As Integer
        Get
            Return mPort
        End Get
        Set(ByVal Value As Integer)
            mPort = Value
        End Set
    End Property

    Public ReadOnly Property HostName() As String
        Get
            Return mHostName
        End Get
    End Property

    Public Property HostIP() As String
        Get
            Return mHostIP
        End Get
        Set(ByVal Value As String)
            mHostIP = Value
        End Set
    End Property

    Public ReadOnly Property State() As eState
        Get
            Return mState
        End Get
    End Property

    Public Property ListenInterval() As Integer
        Get
            Return mListenerTimer.Interval
        End Get
        Set(ByVal Value As Integer)
            mListenerTimer.Interval = Value
        End Set
    End Property

    Public Property DataInterval() As Integer
        Get
            Return mDataTimer.Interval
        End Get
        Set(ByVal Value As Integer)
            mDataTimer.Interval = Value
        End Set
    End Property
#End Region

#Region "Functions"
    Private Sub ListenerTimerHandler(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mListenerTimer.Tick
        If mState = eState.Listening Then
            If mListener.Pending Then
                mListenerTimer.Stop()
                RaiseEvent IncomingConnection()
            End If
        End If
    End Sub

    Private Sub DataTimerHandler(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mDataTimer.Tick
        If mState = eState.Connected Then
            Dim bytes() As Byte
            Dim length, readable As Integer
            readable = mSocket.Available

            If readable > 0 Then
                ReDim bytes(readable - 1)
                length = mSocket.Receive(bytes)
                RaiseEvent DataArrival(bytes, length)
            Else
                If mSocket.Poll(1, SelectMode.SelectRead) Then
                    Me.Close()
                    RaiseEvent Disconnected()
                End If
            End If
        End If
    End Sub

    Public Sub Accept()
        mSocket = mListener.AcceptSocket
        mState = eState.Connected
        RaiseEvent Connected()
        mDataTimer.Start()
    End Sub

    Public Sub Listen(Optional ByVal pPort As Integer = 0)

        If mState = eState.Closed Then
            Dim ipadd As IPAddress
            'get the local ip address and port no
            ipadd = IPAddress.Parse(LocalIP)
            If pPort > 0 Then
                mPort = pPort
            End If

            'create new instance of listener
            If Not (mListener Is Nothing) Then
                mListener = Nothing
            End If
            mListener = New TcpListener(ipadd, mPort)
            mListener.Start()

            mState = eState.Listening
            mListenerTimer.Start()

        Else
            Throw New Exception("Socket must close before start listening.")
            Me.Close()
        End If
    End Sub

    Public Sub Connect(Optional ByVal pHostName As String = "", Optional ByVal pPort As Integer = 0)
        If pHostName.Trim <> "" Then mHostName = pHostName
        If pPort > 0 Then mPort = pPort

        'error might occur, let the calling application handle the error
        Try
            Dim ipep As IPEndPoint

            If IsIPAddress(mHostName) Then
                ipep = New IPEndPoint(IPAddress.Parse(mHostName), mPort)
            Else
                ipep = New IPEndPoint(IPAddress.Parse(Dns.Resolve(mHostName).AddressList(0).ToString), mPort)
            End If
            mSocket = New Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp)
            mSocket.Connect(CType(ipep, EndPoint))
            mState = eState.Connected
            RaiseEvent Connected()
            mDataTimer.Start()
        Catch ex As Exception
            Me.Close()
            Throw ex
        End Try
    End Sub

    Public Sub SendData(ByVal pBytes() As Byte)
        mSocket.Send(pBytes)
    End Sub

    Public Sub SendData(ByVal pString As String)
        Dim bytes() As Byte = System.Text.Encoding.ASCII.GetBytes(pString)
        mSocket.Send(bytes)
    End Sub

    Private Function IsIPAddress(ByVal pString As String) As Boolean
        Dim i, num As Byte
        Dim numbers(3) As String        

        numbers = pString.Split("."c)

        If numbers.GetUpperBound(0) <> 3 Then
            Return False
        End If

        For i = 0 To 3
            Try
                num = CType(numbers(i), Byte)
            Catch ex As Exception
                Return False
            End Try
        Next

        Return True
    End Function

    Public Sub Close()
        mState = eState.Closed
        mDataTimer.Stop()
        mListenerTimer.Stop()
        If Not (mSocket Is Nothing) Then mSocket.Close()
        If Not (mListener Is Nothing) Then mListener.Stop()
    End Sub
#End Region

End Class